@@ -1,9 +1,9 @@
 /*
- *  $Id: fselect.c,v 1.105 2019/08/06 00:38:38 tom Exp $
+ *  $Id: fselect.c,v 1.116 2021/03/05 00:47:39 tom Exp $
  *
  *  fselect.c -- implements the file-selector box
  *
- *  Copyright 2000-2018,2019	Thomas E. Dickey
+ *  Copyright 2000-2020,2021	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #include <sys/types.h>
@@ -292,6 +292,7 @@ fix_arrows(LIST * list)
 			: KEY_NPAGE));
     }
 }
+
 #else
 #define fix_arrows(list)	/* nothing */
 #endif
@@ -362,18 +363,29 @@ match(char *name, LIST * d_list, LIST * f_list, MATCH * match_list)
     size_t test_len = strlen(test);
     char **matches = dlg_malloc(char *, (size_t) (d_list->length + f_list->length));
     size_t data_len = 0;
-    int i;
-    for (i = 2; i < d_list->length; i++) {
-	if (strncmp(test, d_list->data[i], test_len) == 0) {
-	    matches[data_len++] = d_list->data[i];
+
+    if (matches != 0) {
+	int i;
+	char **new_ptr;
+
+	for (i = 2; i < d_list->length; i++) {
+	    if (strncmp(test, d_list->data[i], test_len) == 0) {
+		matches[data_len++] = d_list->data[i];
+	    }
+	}
+	for (i = 0; i < f_list->length; i++) {
+	    if (strncmp(test, f_list->data[i], test_len) == 0) {
+		matches[data_len++] = f_list->data[i];
+	    }
+	}
+	if ((new_ptr = dlg_realloc(char *, data_len + 1, matches)) != 0) {
+	    matches = new_ptr;
+	} else {
+	    free(matches);
+	    matches = 0;
+	    data_len = 0;
 	}
     }
-    for (i = 0; i < f_list->length; i++) {
-	if (strncmp(test, f_list->data[i], test_len) == 0) {
-	    matches[data_len++] = f_list->data[i];
-	}
-    }
-    matches = dlg_realloc(char *, data_len + 1, matches);
     match_list->data = matches;
     match_list->length = (int) data_len;
 }
@@ -396,6 +408,7 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
 
     match(name, d_list, f_list, &match_list);
     if (match_list.length == 0) {
+	free(match_list.data);
 	*buff_ptr = NULL;
 	return 0;
     }
@@ -412,6 +425,7 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
 	}
     } else {
 	int j;
+	char *next;
 
 	for (i = 0; i < test_len; i++) {
 	    char test_char = test[i];
@@ -427,7 +441,13 @@ complete(char *name, LIST * d_list, LIST * f_list, char **buff_ptr)
 	    } else
 		break;
 	}
-	buff = dlg_realloc(char, i + 1, buff);
+	next = dlg_realloc(char, i + 1, buff);
+	if (next == NULL) {
+	    free(buff);
+	    *buff_ptr = NULL;
+	    return 0;
+	}
+	buff = next;
     }
     free_match(&match_list);
     buff[i] = '\0';
@@ -632,10 +652,8 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 #ifdef KEY_RESIZE
   retry:
 #endif
-    dlg_auto_size(title, (char *) 0, &height, &width, 6, 25);
-    height += MIN_HIGH + min_items;
-    if (width < min_wide)
-	width = min_wide;
+    dlg_auto_size(title, "", &height, &width, MIN_HIGH + min_items, min_wide);
+
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
 
@@ -659,13 +677,12 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
     tbox_y = height - (BTN_HIGH * 2) + MARGIN;
     tbox_x = (width - tbox_width) / 2;
 
-    w_text = derwin(dialog, tbox_height, tbox_width, tbox_y, tbox_x);
+    w_text = dlg_der_window(dialog, tbox_height, tbox_width, tbox_y, tbox_x);
     if (w_text == 0) {
 	result = DLG_EXIT_ERROR;
 	goto finish;
     }
 
-    (void) keypad(w_text, TRUE);
     dlg_draw_box(dialog, tbox_y - MARGIN, tbox_x - MARGIN,
 		 (2 * MARGIN + 1), tbox_width + (MARGIN + EXT_WIDE),
 		 menubox_border_attr, menubox_border2_attr);
@@ -686,13 +703,12 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
     dbox_y = (2 * MARGIN + 1);
     dbox_x = tbox_x;
 
-    w_work = derwin(dialog, dbox_height, dbox_width, dbox_y, dbox_x);
+    w_work = dlg_der_window(dialog, dbox_height, dbox_width, dbox_y, dbox_x);
     if (w_work == 0) {
 	result = DLG_EXIT_ERROR;
 	goto finish;
     }
 
-    (void) keypad(w_work, TRUE);
     (void) mvwaddstr(dialog, dbox_y - (MARGIN + 1), dbox_x - MARGIN, d_label);
     dlg_draw_box(dialog,
 		 dbox_y - MARGIN, dbox_x - MARGIN,
@@ -707,13 +723,12 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 	fbox_y = dbox_y;
 	fbox_x = tbox_x + dbox_width + (2 * MARGIN);
 
-	w_work = derwin(dialog, fbox_height, fbox_width, fbox_y, fbox_x);
+	w_work = dlg_der_window(dialog, fbox_height, fbox_width, fbox_y, fbox_x);
 	if (w_work == 0) {
 	    result = DLG_EXIT_ERROR;
 	    goto finish;
 	}
 
-	(void) keypad(w_work, TRUE);
 	(void) mvwaddstr(dialog, fbox_y - (MARGIN + 1), fbox_x - MARGIN, f_label);
 	dlg_draw_box(dialog,
 		     fbox_y - MARGIN, fbox_x - MARGIN,
@@ -861,6 +876,10 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 	    case DLGK_ENTER:
 		result = (state > 0) ? dlg_enter_buttoncode(state) : DLG_EXIT_OK;
 		continue;
+	    case DLGK_LEAVE:
+		if (state >= 0)
+		    result = dlg_ok_buttoncode(state);
+		break;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
 		dlg_will_resize(dialog);
@@ -871,10 +890,9 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 		*current = 0;
 		resized = TRUE;
 		/* repaint */
-		dlg_clear();
-		dlg_del_window(dialog);
-		refresh();
-		dlg_mouse_free_regions();
+		free_list(&d_list, FALSE);
+		free_list(&f_list, FALSE);
+		_dlg_resize_cleanup(dialog);
 		goto retry;
 #endif
 	    default:
@@ -913,14 +931,12 @@ dlg_fselect(const char *title, const char *path, int height, int width, int dsel
 		first = FALSE;
 		state = sTEXT;
 	    }
-	} else if (state >= 0 &&
-		   (code = dlg_char_to_button(key, buttons)) >= 0) {
+	} else if ((code = dlg_char_to_button(key, buttons)) >= 0) {
 	    result = dlg_ok_buttoncode(code);
 	    break;
 	}
     }
-    dlg_add_separator();
-    dlg_add_last_key(-1);
+    AddLastKey();
 
     dlg_unregister_window(w_text);
     dlg_del_window(dialog);
